@@ -225,7 +225,7 @@ int ssh_userauth_none(ssh_session session, const char *username) {
 
   if (username == NULL) {
     if (session->username == NULL) {
-      if (ssh_options_set(session, SSH_OPTIONS_USER, NULL) < 0) {
+      if (ssh_options_apply(session) < 0) {
         leave_function();
         return rc;
       }
@@ -331,7 +331,7 @@ int ssh_userauth_offer_pubkey(ssh_session session, const char *username,
 
   if (username == NULL) {
     if (session->username == NULL) {
-      if (ssh_options_set(session, SSH_OPTIONS_USER, NULL) < 0) {
+      if (ssh_options_apply(session) < 0) {
         leave_function();
         return rc;
       }
@@ -443,7 +443,7 @@ int ssh_userauth_pubkey(ssh_session session, const char *username,
 
   if (username == NULL) {
     if (session->username == NULL) {
-      if (ssh_options_set(session, SSH_OPTIONS_USER, NULL) < 0) {
+      if (ssh_options_apply(session) < 0) {
         leave_function();
         return rc;
       }
@@ -561,7 +561,7 @@ int ssh_userauth_agent_pubkey(ssh_session session, const char *username,
 
   if (username == NULL) {
     if (session->username == NULL) {
-      if (ssh_options_set(session, SSH_OPTIONS_USER, NULL) < 0) {
+      if (ssh_options_apply(session) < 0) {
         leave_function();
         return rc;
       }
@@ -688,7 +688,7 @@ int ssh_userauth_password(ssh_session session, const char *username,
 
   if (username == NULL) {
     if (session->username == NULL) {
-      if (ssh_options_set(session, SSH_OPTIONS_USER, NULL) < 0) {
+      if (ssh_options_apply(session) < 0) {
         leave_function();
         return rc;
       }
@@ -777,7 +777,6 @@ error:
  *
  * @see ssh_userauth_kbdint()
  * @see ssh_userauth_password()
- * @see ssh_options_set()
  */
 int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
   struct ssh_iterator *it;
@@ -860,16 +859,14 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
   } /* if agent is running */
 #endif
 
+
   for (it = ssh_list_get_iterator(session->identity);
        it != NULL;
        it = it->next) {
-    char *privkey_file = NULL;
+    const char *privkey_file = it->data;
     int privkey_open = 0;
 
-    privkey_file = dir_expand_dup(session, it->data, 1);
-    if (privkey_file == NULL) {
-      continue;
-    }
+    privkey = NULL;
 
     ssh_log(session, SSH_LOG_PROTOCOL, "Trying to read privatekey %s", privkey_file);
 
@@ -883,7 +880,6 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
         ssh_log(session, SSH_LOG_RARE,
           "Reading private key %s failed (bad passphrase ?)",
           privkey_file);
-        SAFE_FREE(privkey_file);
         leave_function();
         return SSH_AUTH_ERROR;
       }
@@ -891,7 +887,6 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
 
       pubkey = publickey_from_privatekey(privkey);
       if (pubkey == NULL) {
-        SAFE_FREE(privkey_file);
         privatekey_free(privkey);
         ssh_set_error_oom(session);
         leave_function();
@@ -902,7 +897,6 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
       type = pubkey->type;
       publickey_free(pubkey);
       if (pubkey_string == NULL) {
-        SAFE_FREE(privkey_file);
         ssh_set_error_oom(session);
         leave_function();
         return SSH_AUTH_ERROR;
@@ -911,7 +905,6 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
       len = strlen(privkey_file) + 5;
       publickey_file = malloc(len);
       if (publickey_file == NULL) {
-        SAFE_FREE(privkey_file);
         ssh_set_error_oom(session);
         leave_function();
         return SSH_AUTH_ERROR;
@@ -924,13 +917,11 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
       }
       SAFE_FREE(publickey_file);
     } else if (rc < 0) {
-      SAFE_FREE(privkey_file);
       continue;
     }
 
     rc = ssh_userauth_offer_pubkey(session, NULL, type, pubkey_string);
     if (rc == SSH_AUTH_ERROR){
-      SAFE_FREE(privkey_file);
       string_free(pubkey_string);
       ssh_log(session, SSH_LOG_RARE, "Publickey authentication error");
       leave_function();
@@ -938,7 +929,6 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
     } else {
       if (rc != SSH_AUTH_SUCCESS){
         ssh_log(session, SSH_LOG_PROTOCOL, "Publickey refused by server");
-        SAFE_FREE(privkey_file);
         string_free(pubkey_string);
         continue;
       }
@@ -953,7 +943,6 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
         ssh_log(session, SSH_LOG_RARE,
             "Reading private key %s failed (bad passphrase ?)",
             privkey_file);
-        SAFE_FREE(privkey_file);
         string_free(pubkey_string);
         continue; /* continue the loop with other pubkey */
       }
@@ -961,7 +950,6 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
 
     rc = ssh_userauth_pubkey(session, NULL, pubkey_string, privkey);
     if (rc == SSH_AUTH_ERROR) {
-      SAFE_FREE(privkey_file);
       string_free(pubkey_string);
       privatekey_free(privkey);
       leave_function();
@@ -970,7 +958,6 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
       if (rc != SSH_AUTH_SUCCESS){
         ssh_log(session, SSH_LOG_FUNCTIONS,
             "The server accepted the public key but refused the signature");
-        SAFE_FREE(privkey_file);
         string_free(pubkey_string);
         privatekey_free(privkey);
         continue;
@@ -980,7 +967,6 @@ int ssh_userauth_autopubkey(ssh_session session, const char *passphrase) {
     /* auth success */
     ssh_log(session, SSH_LOG_PROTOCOL,
         "Successfully authenticated using %s", privkey_file);
-    SAFE_FREE(privkey_file);
     string_free(pubkey_string);
     privatekey_free(privkey);
 
@@ -1349,7 +1335,7 @@ int ssh_userauth_kbdint(ssh_session session, const char *user,
     /* first time we call. we must ask for a challenge */
     if (user == NULL) {
       if ((user = session->username) == NULL) {
-        if (ssh_options_set(session, SSH_OPTIONS_USER, NULL) < 0) {
+        if (ssh_options_apply(session) < 0) {
           leave_function();
           return SSH_AUTH_ERROR;
         } else {
@@ -1381,7 +1367,7 @@ int ssh_userauth_kbdint(ssh_session session, const char *user,
 
   /*
    * If we are at this point, it ss because session->kbdint exists.
-   * It means the user has set some informations there we need to send
+   * It means the user has set some information there we need to send
    * the server and then we need to ack the status (new questions or ok
    * pass in).
    */
